=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2009-10-15 20:04:37 +0000
+++ bzrlib/tests/test_ui.py	2009-11-15 22:10:30 +0000
@@ -50,6 +50,7 @@
     NullProgressView,
     TextProgressView,
     TextUIFactory,
+    TextUIOutputStream,
     )
 
 
@@ -253,6 +254,29 @@
             pb.finished()
 
 
+class TestTextUIOutputStream(TestCase):
+    """Tests for output stream that synchronizes with progress bar."""
+
+    def test_output_clears_terminal(self):
+        stdout = StringIO()
+        stderr = StringIO()
+        clear_calls = []
+
+        uif = TextUIFactory(None, stdout, stderr)
+        uif.clear_term = lambda: clear_calls.append('clear')
+
+        stream = TextUIOutputStream(uif, uif.stdout)
+        stream.write("Hello world!\n")
+        stream.write("there's more...\n")
+        
+        self.assertEqual(stdout.getvalue(),
+            "Hello world!\n"
+            "there's more...\n")
+        self.assertEqual(['clear', 'clear'],
+            clear_calls)
+
+
+
 class UITests(tests.TestCase):
 
     def test_progress_construction(self):

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2009-09-23 06:29:46 +0000
+++ bzrlib/ui/text.py	2009-11-15 22:10:30 +0000
@@ -365,3 +365,25 @@
             self._bytes_since_update = 0
             self._last_transport_msg = msg
             self._repaint()
+
+
+class TextUIOutputStream(object):
+    """Decorates an output stream so that the terminal is cleared before writing.
+
+    This is supposed to ensure that the progress bar does not conflict with bulk
+    text output.
+    """
+    # XXX: this does not handle the case of writing part of a line, then doing
+    # progress bar output: the progress bar will probably write over it.
+    # one option is just to buffer that text until we have a full line;
+    # another is to save and restore it
+
+    # XXX: might need to wrap more methods
+
+    def __init__(self, ui_factory, wrapped_stream):
+        self.ui_factory = ui_factory
+        self.wrapped_stream = wrapped_stream
+
+    def write(self, to_write):
+        self.ui_factory.clear_term()
+        self.wrapped_stream.write(to_write)

